----------------------------------------------------------------------------------
-- Company: Digilent RO
-- Engineer: Mircea Dabacan
-- 
-- Create Date:    18:00:57 03/02/2008 
-- Design Name: 
-- Module Name:    Hb5RefProj - Mixt (Structural and Behavioral)
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: This project demonstrates the use of the Digilent HB5 Reference 
--              Component, HB5RefComp, with a DC motor.
--              The SA and SB hall sensors signals are not used, no loop control
--              is implemented.
--              The project can also be used wit the Pmod HB3.
--              The above component is instantiated to control a Pmod HB5
--              The adjustBcd component provides a BCD value, adjustable with
--              btnFaster and btnSlower.
--              This value is used to set the duty factor of pinEn. The value is 
--              also shown on the 7 segment display.
--              swDir sets the direction for the motor, after being secured in
--              the HB5RefComp. The direction is also shown on the 7 segment 
--              display as "-" (for minus) or nothing (for plus).
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity Hb5RefProj is
    Port ( ck : in  STD_LOGIC;         -- system clock (50MHz)
           btnFaster : in  STD_LOGIC;  -- rise Duty Factor command
           btnSlower : in  STD_LOGIC;  -- lower Duty Factor command
           swDir : in  STD_LOGIC;      -- Direction command
           pinEn : out  STD_LOGIC;     -- HB5(HB3) pin En
           pinDir : out  STD_LOGIC;    -- HB5(HB3) pin Dir
           seg: out std_logic_vector(6 downto 0); -- segment outputs
           an : out std_logic_vector(3 downto 0));-- anode select signals
end Hb5RefProj;

architecture Mixt of Hb5RefProj is

	COMPONENT adjustBcd
	PORT(
		ck : IN std_logic;
		bitCountUp : IN std_logic;
		bitCountDown : IN std_logic;          
		vecBcdOut : OUT std_logic_vector(7 downto 0)
		);
	END COMPONENT;

	COMPONENT HB5RefComp
	PORT(
		ck : IN std_logic;
		bitDirIn : IN std_logic;
		vecDfIn : IN std_logic_vector(7 downto 0);          
		bitEnOut : OUT std_logic;
		bitDirOut : OUT std_logic
		);
	END COMPONENT;

  signal vecDf: STD_LOGIC_VECTOR (7 downto 0) := x"00"; -- Duty Factor for PWM
  signal HEX : std_logic_vector(3 downto 0);  -- a digit for 7 seg display
  signal cnt : std_logic_vector(18 downto 0):="0000000000000000000";-- divider for 7SD

begin

	Inst_adjustBcd: adjustBcd PORT MAP(
		ck => ck,
		bitCountUp => btnFaster,
		bitCountDown => btnSlower,
		vecBcdOut => vecDf
	);

	Inst_HB5RefComp: HB5RefComp PORT MAP(
		ck => ck,
		bitDirIn => swDir,
		vecDfIn => vecDf,
		bitEnOut => pinEn,
		bitDirOut => pinDir
	);

  -- displaying the BCD value 
div: process (ck)
 begin
  if rising_edge(ck) then
   cnt <=cnt + "0000000000000000001";
  end if;
 end process;

with cnt(18 downto 17) select
 hex <= vecDf(3 downto 0) when "00",
        vecDf(7 downto 4) when "01",
        "111" & swDir     when "10",  -- "1110" for positive, "1111" for negative
        "1110" when others; -- MSD off

with cnt(18 downto 17) select
 an <= "1110" when "00",
       "1101" when "01",
       "1011" when "10",
       "0111" when others; 

--HEX-to-seven-segment decoder
-- 
-- segment encoding
--      0
--     ---  
--  5 |   | 1
--     ---   <- 6
--  4 |   | 2
--     ---
--      3
   
    with HEX select
   seg<= "1000000" when "0000",   --0
         "1111001" when "0001",   --1
         "0100100" when "0010",   --2
         "0110000" when "0011",   --3
         "0011001" when "0100",   --4
         "0010010" when "0101",   --5
         "0000010" when "0110",   --6
         "1111000" when "0111",   --7
         "0000000" when "1000",   --8
         "0010000" when "1001",   --9
         "0111111" when "1111",   -- minus sign
         "1111111" when others;   -- nothing, for plus sign
 


end Mixt;

