----------------------------------------------------------------------------------
-- Company: Digilent RO
-- Engineer: Mircea Dabacan
-- 
-- Create Date:    17:42:05 03/02/2008 
-- Design Name: 
-- Module Name:    adjustBcd - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: Up/Down BCD counter with predivider and with no rollover.
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity adjustBcd is
    Port ( ck : in  STD_LOGIC;
           bitCountUp : in  STD_LOGIC;
           bitCountDown : in  STD_LOGIC;
           vecBcdOut : out  STD_LOGIC_VECTOR (7 downto 0));
end adjustBcd;

architecture Behavioral of adjustBcd is

  constant intPreDiv: integer := 5000000; -- predivision factor
  -- with intPreDiv = 5000000:
          -- frequency(ckInt) = 50MHz/5000000 = 10Hz

  signal cntPreDiv: integer range 0 to intPreDiv - 1 := 0; --counter for predivider
  signal ckInt: STD_LOGIC; -- Internal clock
  signal cntUpDown: STD_LOGIC_VECTOR (7 downto 0) := x"00"; -- counter for PWM

begin

  preDiv: process(ck)  -- clock predivision
  begin
    if ck'event and ck = '1' then
	   if cntPreDiv = intPreDiv - 1 then
		  cntPreDiv <= 0;
		  ckInt <= '1';
		else
        cntPreDiv <= cntPreDiv + 1;
		  ckInt <= '0';
      end if;
    end if;
  end process;	 

  UpDownCount: process(ckInt)  -- UpDown counter
  begin
    if ckInt'event and ckInt = '1' then
	   if cntUpDown /= x"99"  and bitCountUp = '1' then  -- count up
		  if cntUpDown(3 downto 0) = x"9" then
		    cntUpDown(3 downto 0) <= x"0";
		    cntUpDown(7 downto 4) <= cntUpDown(7 downto 4) + x"1";
		  else  
			 cntUpDown(3 downto 0) <= cntUpDown(3 downto 0) + x"1";
        end if;
	   elsif cntUpDown /= x"00"  and bitCountDown = '1' then  -- count down
		  if cntUpDown(3 downto 0) = x"0" then
		    cntUpDown(3 downto 0) <= x"9";
		    cntUpDown(7 downto 4) <= cntUpDown(7 downto 4) - x"1";
		  else  
			 cntUpDown(3 downto 0) <= cntUpDown(3 downto 0) - x"1";
        end if;
		else
        null;   -- no count
      end if;
    end if;
  end process;	 

  vecBcdOut <= cntUpDown;

end Behavioral;

